#ifndef PORT_MGR_H
#define PORT_MGR_H

/*
 * $Id: PortMgr.h,v 1.2 1999/04/15 20:36:09 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "LevelStat.h"
#include "BitMask.h"

class Socket;

// manages allocation of network ports
// see EphPortMgr and ExpPortMgr for concrete classes
class PortMgr {
	public:
		PortMgr();
		virtual ~PortMgr() {}

		int bind(Socket &s); // returns port number
		void release(int port, bool good);

		// notify about ports allocated outside of the algorithm
		void reserve(int port) { theResPorts.beSet(port); }
		void cancel(int port) { theResPorts.beClear(port); }

	protected:
		virtual int allocPort(Socket &s) = 0;
		virtual void freePort(int port, bool good) = 0;

	protected:
		BitMask theResPorts;   // reserved ports
		LevelStat theBoundLvl; // bounded ports level
};

#endif
