#ifndef WORLD_CLASS_H
#define WORLD_CLASS_H

/*
 * $Id: World.h,v 1.1.1.7 1999/02/22 23:50:21 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "Time.h"
#include "Size.h"
#include "String.h"
#include "Array.h"
#include "Oid.h"

class ostream;

class PopModel;
class ChbModel;
class TmlModel;
class HitModel;

class World {
	public:
		World();
		~World();

		const String &type() const { return theType; }
		const String &id() const { return theId; }
		const char *url(int oid) const { return (oid > 0 && oid <= theUrls.count()) ? theUrls[oid-1] : 0; }
		int oid(int offset, OidRec &rec) { return theObjIds.count() ? theObjIds[offset % theObjIds.count()] : genOid(rec); }
		int uid(int offset) const { return theUnqIds.count() ? theUnqIds[offset % theUnqIds.count()] : offset+1; }

		int genOid(OidRec &rec);
		void noteResponse(const OidRec &) {} // looks like we do not care

		void type(const String &aType) { theType = aType; }
		void id(const String &anId) { theId = anId; }

		void prefill(int count);
		bool loadUrls(const char *fname);
		bool genObjIds(int count, int order, bool unique);

		ostream &report(ostream &os) const;

	protected:
		int repeatOid(int lastId, bool cachable);

	public:
		PopModel *thePopMdl;
		ChbModel *theChbMdl;
		TmlModel *theTmlMdl;
		HitModel *theHitMdl;

	protected:
		String theType;      // universe id, sort of
		String theId;        // world id (to generate unique worlds)

		Array<char*> theUrls;// pre-loaded urls if any
		Array<int> theObjIds;// pre-generated object IDs if any
		Array<int> theUnqIds;// pre-generated unique IDs if any

		int theLastChbId;
		int theLastUncId;

		bool replanOid;      // used internally by genOid routines
};

extern World TheWorld;

#endif
